<?php

namespace Drupal\entity_agree\Plugin\views;

/**
 * Trait form configuring an agreement in the plugin.
 */
trait AgreementViewsPluginTrait {

  /**
   * The agreement entity.
   *
   * Use ::getAgreementEntity to access.
   *
   * @var \Drupal\Core\Entity\RevisionableInterface
   */
  protected $agreementEntity;

  /**
   * The agreement manager.
   *
   * @var \Drupal\entity_agree\AgreementManager
   */
  protected $agreementManager;

  /**
   * Entity repository.
   *
   * @var \Drupal\Core\Entity\EntityRepositoryInterface
   */
  protected $entityRepository;

  /**
   * Get the entity from configured options.
   *
   * @return \Drupal\Core\Entity\RevisionableInterface|false
   *   The agreement entity, or FALSE if not available.
   */
  protected function getAgreementEntity($option = 'agreement') {
    if ($this->agreementEntity === NULL) {
      $values = explode(':', $this->options[$option]);
      $entity_type_id = array_shift($values);
      $entity_uuid = array_shift($values);
      if ($entity_type_id && $entity_uuid) {
        $agreement = $this->entityRepository->loadEntityByUuid($entity_type_id, $entity_uuid);
      }
      else {
        $agreement = FALSE;
      }
      $this->agreementEntity = $agreement;
    }
    return $this->agreementEntity;
  }

  /**
   * Validation handling for agreement option.
   */
  public function validateAgreement($option = 'agreement') {
    if (!$this->getAgreementEntity()) {
      if (empty($this->options[$option])) {
        return [$this->t('Agreement not configured.')];
      }
      return [$this->t('Agreement not available.')];
    }
    return [];
  }

  /**
   * Set agreement option definition.
   *
   * @return array
   *   Option definitions with the agreement set.
   */
  protected function defineAgreementOption(array $options) {
    return $options + [
      'agreement' => ['default' => ''],
    ];
  }

  /**
   * Build a form element for selecting an agreement option.
   *
   * @param mixed $default_value
   *   The default value.
   *
   * @return array
   *   The form element.
   */
  protected function buildAgreementOptionElement($default_value) {
    $options = [];
    $agreements = $this->agreementManager->loadEntities($this->agreementManager->getAgreementRevisions());
    foreach ($agreements as $agreement_entities) {
      foreach ($agreement_entities as $entity) {
        $value = "{$entity->getEntityTypeId()}:{$entity->uuid()}";
        $options[(string) $entity->getEntityType()->getLabel()][$value] = $entity->label();
      }
    }

    return [
      '#type' => 'select',
      '#title' => $this->t('Agreement'),
      '#options' => $options,
      '#default_value' => $default_value,
    ];
  }

  /**
   * Set complete and pending text option definitions.
   *
   * @return array
   *   Option definitions with the complete and pending text set.
   */
  protected function defineStatusOptions(array $options) {
    return $options + [
      'complete_text' => ['default' => 'Agreed'],
      'pending_text' => ['default' => 'Pending'],
    ];
  }

  /**
   * Build a form element option for a text option.
   *
   * @param string $title
   *   The element title.
   * @param mixed $default_value
   *   The default value.
   *
   * @return array
   *   The form element.
   */
  protected function buildStatusOptionElement($title, $default_value) {
    return [
      '#type' => 'textfield',
      '#title' => $title,
      '#default_value' => $default_value,
    ];
  }

}
